unit MainStringsEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, SynHighlighterSQL, SynEditHighlighter,
  SynHighlighterPas, SynEdit, SynEditPythonBehaviour, SynAutoCorrect;

type

  TTypeWords = (twNone, twDelphi, twSql);

  TfMainStringsEditor = class(TForm)
    Panel1: TPanel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    RG: TRadioGroup;
    Edit: TSynEdit;
    Pas: TSynPasSyn;
    SQL: TSynSQLSyn;
    Lb: TLabel;
    BitBtn3: TBitBtn;
    OpenDialog: TOpenDialog;
    procedure RGClick(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EditMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure EditEnter(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
  private
    FTypeSyntax: TTypeWords;
    procedure SetTypeSyntax(const Value: TTypeWords);
    { Private declarations }
  public
    { Public declarations }

    Property TypeSyntax: TTypeWords read FTypeSyntax write SetTypeSyntax;
    Procedure CurPos();
  end;

  Function ShowEditor(Text: String; TypeSyntax: TTypeWords = twNone): String;

var
  fMainStringsEditor: TfMainStringsEditor;

implementation

{$R *.DFM}


Function ShowEditor(Text: String; TypeSyntax: TTypeWords = twNone): String;
var
  mr: TModalResult;
begin
  Result:= Text;
  try
    fMainStringsEditor:= TfMainStringsEditor.Create(Application);
    fMainStringsEditor.Edit.Lines.Text:= Text;
    case TypeSyntax of
      twNone: fMainStringsEditor.RG.ItemIndex:= 2;
      twSql: fMainStringsEditor.RG.ItemIndex:= 0;
      twDelphi: fMainStringsEditor.RG.ItemIndex:= 1;
    else ;
    end;

    if fMainStringsEditor.ShowModal = mrOk then
    begin
      Result:= fMainStringsEditor.Edit.Lines.Text;
    end;
  finally
    FreeAndNil(fMainStringsEditor);
  end;

end;

procedure TfMainStringsEditor.CurPos;
var
  s: String;

begin
  s:= 'pos  -  ' + IntToStr(Edit.CaretY);
  s:= s + ' / ' + IntToStr(Edit.CaretX);
  Lb.Caption:= s;
end;

procedure TfMainStringsEditor.RGClick(Sender: TObject);
begin
  case RG.ItemIndex of
    0: TypeSyntax:= twSql;
    1: TypeSyntax:= twDelphi;
    2: TypeSyntax:= twNone;
  end;
end;

procedure TfMainStringsEditor.SetTypeSyntax(const Value: TTypeWords);
begin
  FTypeSyntax := Value;
  case FTypeSyntax of
    twNone: Edit.Highlighter:= Nil;
    twDelphi: Edit.Highlighter:= Pas;
    twSql: Edit.Highlighter:= SQL;
  end;
  if Edit.Highlighter = Nil then
    OpenDialog.Filter:= 'All Files |*.*'
  else
    OpenDialog.Filter:= Edit.Highlighter.DefaultFilter;
end;

procedure TfMainStringsEditor.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  CurPos();
end;

procedure TfMainStringsEditor.EditMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  CurPos();
end;

procedure TfMainStringsEditor.EditEnter(Sender: TObject);
begin
  CurPos();
end;

procedure TfMainStringsEditor.EditKeyPress(Sender: TObject; var Key: Char);
begin
  CurPos();
end;

procedure TfMainStringsEditor.FormShow(Sender: TObject);
begin
  Edit.SetFocus;
end;

procedure TfMainStringsEditor.BitBtn3Click(Sender: TObject);
var
  FileName: String;
begin
  if not OpenDialog.Execute then exit;
  FileName:= OpenDialog.FileName;
  Edit.Lines.LoadFromFile(FileName);
end;

end.
