unit PS;

interface

uses
  SysUtils, Classes, Forms, Dialogs, Menus,
  ifps3utl, ifpscomp, ifps3, ifps3lib_std, ifps3lib_stdr,
  ifps3common, ifpidelphi, ifpidelphiruntime, ifpidll2,
  ifpiir_SysImp, ifpii_SysImp, Math;


type


  TUserFunction = Procedure (const Name: String; var Declare: String; var ProcPtr: Pointer) of object;
  TUserVar = Procedure (VarName: String; var VarValue: Variant) of object;

  TPasFunction = Record
    Declare: String; //   
    Name: String; //  
    Description: String; //  
    ProcPtr: Pointer; //   
  end;
  TPasFunctionList = Array of TPasFunction;

  TPasVar = Record
    Name: String;
    Value: Variant;
  end;
  TPasVarList = Array of TPasVar;

  TPS = class(TComponent)
  private
    FPasComp: TIFPSPascalCompiler;
    FPasExec: TIFPSExec;
    FActive: Boolean;
    FVarName: String;
    FVarValue: String;
    FScript: TStringList;
    FOnUserVar: TUserVar;
    FCompileData: String;
    FCompiled: Boolean;
    FListFunc: TPasFunctionList;
    FScriptStr: String;
    FOnUserFunction: TUserFunction;
    FListVar: TPasVarList;
    FImpListVar: TPasVarList;
    FErrLine: Integer;
    FErrMsg: String;
    FErrPos: Integer;
    FListUserFuction: TStringList;
    FListDelphiFuction: TStringList;
    procedure SetActive(const Value: Boolean);
    procedure SetScript(const Value: TStringList);
    procedure SetVarName(const Value: String);
    procedure SetVarValue(const Value: String);
    procedure SetOnUserVar(const Value: TUserVar);

    Procedure MsgError ();
    procedure ClearError();
    function GetLineError(Position: Integer): Integer;
    procedure SetOnUserFunction(const Value: TUserFunction);
    Function CompileScript(): Boolean;
    Function RunScript(): Boolean;
    function GetVarCount: Integer;
    procedure FillUserFumction(UF: TPasFunction);

    { Private declarations }
  protected
    { Protected declarations }

  public
    { Public declarations }

    Constructor Create (AOwner: TComponent); override;
    destructor  Destroy; override;

    Procedure SetFunction(Declare: String;
                          Name: String;
                          ProcPtr: Pointer;
                          Description: String = '');
    Procedure Activate();
    Procedure DeActivate();
    Function VarByName(VarName: String): Variant;
    Function GetVarIndex(VarName: String): Integer;
    Procedure VarClear();

    property Compiler: TIFPSPascalCompiler read fPasComp;
    property Exec: TIFPSExec read fPasExec;
    property VarList: TPasVarList read FListVar;
    property VarCount: Integer read GetVarCount;
    property ErrLine: Integer read FErrLine;
    property ErrPos: Integer read FErrPos;
    property ErrMsg: String read FErrMsg;
    property ListUserFuction: TStringList read FListUserFuction;

  published
    { Published declarations }
    Property Active: Boolean read FActive write SetActive;
    Property Script: TStringList read FScript write SetScript;
    Property VarName: String read FVarName write SetVarName;
    Property VarValue: String read FVarValue write SetVarValue;

    Property OnUserFunction: TUserFunction read FOnUserFunction write SetOnUserFunction;
    //Property OnUserVar: TUserVar read FOnUserVar write SetOnUserVar;


  end;




  function MyOnUses(Sender: TIFPSPascalCompiler;
                    const Name: string): Boolean;
  function MyExportCheck(Sender: TIFPSPascalCompiler;
                         Proc: PIFPSProcedure;
                         const ProcDecl: string): Boolean;
  Procedure VarExp(VarName: String; VarValue: Variant);
  Procedure VarExpAdd(VarName: String; VarValue: Variant);
  Function VarImp(VarName: String): Variant;
  Function VarImpFloat(VarName: String): Double;
  Function VarImpStr(VarName: String): String;
  Function VarImpInt(VarName: String): Integer;
  Procedure VarClear();
  Procedure SetVar(VarName: String; VarValue: Variant; var ListVar: TPasVarList);


  //procedure Register;

implementation
{$R *.dcr}

//procedure Register;
//begin
  //RegisterComponents('PS', [TPS]);
//end;

var
  aListFunc: TPasFunctionList;
  aListVar, aImpListVar: TPasVarList;
  pPS: TPS;


Procedure VarClear();
begin
  pPS.VarClear;
end;

Function FindVar(Name: String; ListVar: TPasVarList): Integer;
var
  i: Integer;

begin
  Result:= -1;
  Name:= AnsiUpperCase(Name);
  for i := 0 to Length(ListVar) - 1 do
  begin
    if ListVar[i].Name = Name then
    begin
      Result:= i;
      exit;
    end;
  end;
end;

Function VarImp(VarName: String): Variant;
var
  i: Integer;

begin
  Result:= 0;
  i:= FindVar(VarName, aImpListVar);
  if i >= 0 then
    Result:= aImpListVar[i].Value;
end;

Function VarImpFloat(VarName: String): Double;
var
  Value: Variant;
begin
  Result:= 0.00;
  Value:= VarImp(VarName);
  if VarIsNull(Value) then exit;
  if VarType(Value) in [varDouble, varCurrency, varInteger,
                        varSmallint, varSingle, varDate] then
    Result:= Double(Value);
end;

Function VarImpStr(VarName: String): String;
var
  Value: Variant;
begin
  Result:=  '';
  Value:= VarImp(VarName);
  if VarIsNull(Value) then exit;
  Result:= VarToStr(Value);
end;

Function VarImpInt(VarName: String): Integer;
var
  Value: Variant;
  x: Extended;
begin
  Result:= 0;
  Value:= VarImp(VarName);
  if VarIsNull(Value) then exit;
  Result:= Round(VarImpFloat(VarName));
end;

Procedure VarExpAdd(VarName: String; VarValue: Variant);
var
  Value: Variant;
begin
  Value:= VarImp(VarName);
  if VarIsNull(Value) then
    VarExp(VarName, VarValue)
  else
    VarExp(VarName, VarValue + Value);
end;


Procedure VarExp(VarName: String; VarValue: Variant);
begin
  SetVar(VarName, VarValue, aListVar);
  SetVar(VarName, VarValue, aImpListVar);
end;

Procedure SetVar(VarName: String; VarValue: Variant; var ListVar: TPasVarList);
var
  i: Integer;

begin
  i:= FindVar(VarName, ListVar);
  if i = -1 then
  begin
    SetLength(ListVar, Length(ListVar) + 1);
    i:= Length(ListVar) - 1;
    ListVar[i].Name:= AnsiUpperCase(VarName);
  end;
  ListVar[i].Value:= VarValue;
end;


function MyExportCheck(Sender: TIFPSPascalCompiler; Proc: PIFPSProcedure; const ProcDecl: string): Boolean;
begin
  Result := TRue;
end;

function MyOnUses(Sender: TIFPSPascalCompiler; const Name: string): Boolean;
var
  i: Integer;

begin
  if Name = 'SYSTEM' then
  begin
    RegisterStandardLibrary_C(Sender);
    Register_SysUtilsImp_C(Sender);
    for i:= 0 to Length(aListFunc) - 1 do
    begin
      RegisterDelphiFunctionC(Sender, aListFunc[i].Declare);
    end;
    Result := True;
  end
  else begin
    Result:= False;
    Raise Exception.Create(' !' + #13 +
                           ' ' + Name + #13 +
                           ' ');
  end;
end;



{ TPS }

procedure TPS.Activate;
begin
  Active:= True;
end;

procedure TPS.ClearError;
begin
  FErrLine:= 0;
  FErrPos:= 0;
  FErrMsg:= '';
end;

function TPS.CompileScript: Boolean;
begin
  aListFunc:= FListFunc;
  ClearError();
  FCompiled:= FPasComp.Compile(FScript.Text);
  if not FCompiled then
  begin
    FActive:= False;
    MsgError();
  end
  else begin
    FPasComp.GetOutput(FCompileData);
    if not FPasExec.LoadData(FCompileData) then
    begin
      FCompiled:= False;
      FActive:= False;
      Raise Exception.Create('   ');
    end;
  end;
  FActive:= FCompiled;
  Result:= FCompiled;
end;

constructor TPS.Create(AOwner: TComponent);
begin
  ClearError();
  FScript:= TStringList.Create;
  FListUserFuction:= TStringList.Create;
  FListUserFuction.Sorted:= True;
  SetLength(FListFunc,0);
  SetLength(FListVar,0);
  SetLength(FImpListVar,0);
  FPasComp:= TIFPSPascalCompiler.Create();
  FPasExec:= TIFPSExec.Create();

  SetFunction('Procedure VarExp(VarName: string; VarValue: Variant);', 'VarExp', @VarExp);
  SetFunction('Procedure VarExpAdd(VarName: string; VarValue: Variant);', 'VarExpAdd', @VarExpAdd);
  SetFunction('Function VarImp(VarName: String): Variant;', 'VarImp', @VarImp);
  SetFunction('Function VarImpFloat(VarName: String): Extended;', 'VarImpFloat', @VarImpFloat);
  SetFunction('Function VarImpStr(VarName: String): String;', 'VarImpStr', @VarImpStr);
  SetFunction('Function VarImpInt(VarName: String): Integer;', 'VarImpInt', @VarImpInt);
  SetFunction('Procedure VarClear;', 'VarClear', @Ps.VarClear);

  //     
  RegisterStandardLibrary_R(FPasExec);
  Register_SysUtilsImp_R(FPasExec);

  FPasComp.OnExportCheck := MyExportCheck;
  FPasComp.OnUses:= MyOnUses;
  FPasComp.OnExternalProc:= DllExternalProc;
  FPasExec.CallCleanup := True;
  FCompileData:= '';
  FCompiled:= False;
  inherited Create(AOwner);
end;

procedure TPS.DeActivate;
begin
  Active:= False;
end;

destructor TPS.Destroy;
begin
  FScript.Free;
  FPasComp.Free;
  FPasExec.Free;
  inherited Destroy;
end;

procedure TPS.FillUserFumction;

  Function isExists(): Boolean;
  var
    i, count: Integer;
  begin
    Result:= False;
    count:= FListUserFuction.Count - 1;
    for i:= 0 to count do
    begin
      if (Pos(AnsiUpperCase(UF.Name), AnsiUpperCase(FListUserFuction.Strings[i])) > 0) and
         (Pos(AnsiUpperCase(UF.Declare), AnsiUpperCase(FListUserFuction.Strings[i])) > 0) then
        begin
          Result:= True;
          Break;
        end;
    end;
  end;
begin
  if isExists then exit;
  FListUserFuction.Add(UF.Name + '*' + UF.Declare + '*' + UF.Description);
end;

function TPS.GetLineError(Position: Integer): Integer;
var
  i: Integer;
  s: String;
begin
  Result:= 1;
  s:= FScript.Text;
  for i := 1 to Position do
    if s[i] = #13 then
      inc(Result);
end;

function TPS.GetVarCount: Integer;
begin
  Result:= Length(FListVar);
end;

function TPS.GetVarIndex(VarName: String): Integer;
begin
  VarName:= Trim(VarName);
  Result:= FindVar(VarName, FListVar);
end;

procedure TPS.MsgError;
var
  i: Integer;
  S: String;
  Declare: String;
  ProcPtr: Pointer;
  ProcName: String;

begin
  s:= '';
  Declare:= '';
  ProcPtr:= Nil;
  ProcName:= '';

  if FPasComp.MsgCount = 0 then exit;

  for i := 0 to FPasComp.MsgCount - 1 do
  begin

    FErrPos:= FPasComp.Msg[i].Position;
    FErrLine:= GetLineError(FErrPos);
    FErrMsg:= Trim(IFPSMessageToString(FPasComp.Msg[i]));

    s:= '  - ' + IntToStr(Ord(FPasComp.Msg[i].Error)) + #13;
    s:= s + ErrMsg + #13 + #13;
    s:= s + '   - ' + FPasComp.Msg[i].Param + #13;
    s:= s + '     - ' + IntToStr(ErrLine) + #13;
    s:= s + '    - ' + IntToStr(ErrPos) + #13;
    if FPasComp.Msg[i].MessageType = ptError then
    begin
      //            
      if FPasComp.Msg[i].Error = ecUnknownIdentifier then
      begin
        if Assigned(fOnUserFunction) then
        begin
          ProcName:= AnsiUpperCase(Trim(FPasComp.Msg[i].Param));
          FOnUserFunction(ProcName, Declare, ProcPtr);
          if (Trim(Declare) <> '') and (ProcPtr <> Nil) then
          begin
            SetFunction(Declare, ProcName, ProcPtr);
            if CompileScript() then
              exit;
          end;
        end;
      end;
      //--------------------------------------------------------
    end;
  end;
  s:= Trim(s);

  Raise Exception.Create('  !  [' +
                         IntToStr(FPasComp.MsgCount) + ']' + #13 + #13 + s);

end;

function TPS.RunScript: Boolean;
var
  i: Integer;

begin
  Result:= FCompiled;
  ClearError();
  if not FCompiled then exit;
  SetLength(aListVar,0);
  aImpListVar:= FImpListVar;
  pPS:= Self;
  FPasExec.RunScript;
  for i:= 0 to Length(aListVar) - 1 do
    SetVar(aListVar[i].Name, aListVar[i].Value, FImpListVar);
  FListVar:= aListVar;
  SetVarName(FVarName);
  if FPasExec.ExceptionCode <> ENoError then
  begin
    FActive:= False;
    Result:= False;
    FErrPos:= FPasExec.ExceptionPos;
    FErrLine:= GetLineError(FErrPos);
    FErrMsg:= TIFErrorToString(FPasExec.ExceptionCode,
                               FPasExec.ExceptionString) +
                               ' in ' + IntToStr(FPasExec.ExceptionProcNo) +
                               ' at ' + IntToSTr(FErrPos) + #13 + #13 +
                               FPasExec.ExceptionString;
    FErrMsg:= Trim(FErrMsg);                           
    Raise Exception.Create('  !' + #13 + ErrMsg);
  end;
end;

procedure TPS.SetActive(const Value: Boolean);
var
  s: String;

begin

  FVarValue:= '';
  SetLength(aListVar, 0);
  FListVar:= aListVar;
  if FActive = Value then exit;
  FActive := Value;

  //      
  if (FActive) and (Trim(FScript.Text) = '') then
  begin
    FActive:= False;
    FCompileData:= '';
    FCompiled:= False;
    exit;
  end;

  if FActive and FCompiled then
    FCompiled := (StrComp(PChar(FScript.Text), PChar(FScriptStr)) = 0);


  // 
  if (FActive) and (not FCompiled) then
    CompileScript();

  // 
  if FActive then
    if RunScript() then
      FScriptStr:= FScript.Text;

end;



procedure TPS.SetFunction(Declare, Name: String; ProcPtr: Pointer; Description: String = '');
var
  PasFunc: TPasFunction;

begin
  SetLength(FListFunc, Length(FListFunc) + 1);
  PasFunc.Declare:= Declare;
  PasFunc.Name:= AnsiUpperCase(Name);
  PasFunc.ProcPtr:= ProcPtr;
  PasFunc.Description:= Description;
  FListFunc[Length(FListFunc) - 1]:= PasFunc;
  RegisterDelphiFunctionR(FPasExec, PasFunc.ProcPtr,
                          PasFunc.Name, cdRegister);

  PasFunc.Name:= Name;
  FillUserFumction(PasFunc);
end;


procedure TPS.SetOnUserFunction(const Value: TUserFunction);
begin
  FOnUserFunction:= Value;
end;

procedure TPS.SetOnUserVar(const Value: TUserVar);
begin
  FOnUserVar := Value;
end;

procedure TPS.SetScript(const Value: TStringList);
begin
  if FScript.Text <> Value.Text then
  begin
    FActive:= False;
    FCompiled:= False;
    FCompileData:= '';
  end;
  FScript.Assign(Value);
end;

procedure TPS.SetVarName(const Value: String);
var
  i: Integer;

begin
  FVarName := Value;
  FVarValue:= '';
  if FActive then
  begin
    i:= FindVar(Value, FListVar);
    if i >= 0 then
      FVarValue:= string(FListVar[i].Value);
  end;
end;

procedure TPS.SetVarValue(const Value: String);
begin
end;


function TPS.VarByName(VarName: String): Variant;
var
  i: Integer;
begin
  Result:= 0;
  VarName:= Trim(VarName);
  i:= FindVar(VarName, FListVar);
  if i >= 0 then
    Result:= FListVar[i].Value;
end;

procedure TPS.VarClear;
begin
  SetLength(FImpListVar, 0);
  SetLength(FListVar, 0);
  SetLength(aImpListVar, 0);
  SetLength(aListVar, 0);
end;

end.
 